#!/usr/bin/env python

import argparse
import os
import sys
import numpy as np
import pyshellspec

def read_lc(lfile):
    """
    """
    data = np.loadtxt(lfile, dtype=str)

    names = data[:,0]
    pbands = data[:,1]
    errs = data[:,2].astype('float64')

    return names, pbands, errs

def read_if(ifile):
    """

    :param ifile:
    :return:
    """
    return np.loadtxt(ifile, dtype=str)


# constants
ra = 360 * 18/24. + 50 / 60. + 04.79525 / 3600.
dec = 33 + 21 / 60. + 45.6100 / 3600.
#dir_data = '/home/mira/a/betalyr_MYTEST/data_20171126'
dir_data = '/scratch/mira/a/betalyr_MYTEST/data_20171126'

def main():
    """
    Main loop
    :return:
    """
    # read the data
    obs = []

    # first photometry
    dir = os.path.join(dir_data, 'photometry')
    infile = os.path.join(dir, 'lc.lis')
    names, pbands, errs = read_lc(infile)
    for i in range(0, len(names)):
        print names[i]
        obs.append(pyshellspec.LCData(filename=os.path.join(dir, names[i]),
                                      passband=pbands[i],
                                      global_error=errs[i],
                                      delimiter=','))

    # data from MIRC
    dir = os.path.join(dir_data, 'mirc2017')
    infile = os.path.join(dir, 'mirc.ascii.lis')
    names = read_if(infile)
    for i in range(0, len(names)):
        print names[i]
        obs.append(pyshellspec.IFData(filename=os.path.join(dir, names[i]),
                                      location='chara',
                                      ra=ra, dec=dec, 
                                      format='ascii', weight_vis2=0.5, weight_t3amp=0.5))
    # data from NPOI
    dir = os.path.join(dir_data, 'npoi')
    infile = os.path.join(dir, 'npoi.ascii.lis')
    names = read_if(infile)
    for i in range(0, len(names)):
        print names[i]
        obs.append(pyshellspec.IFData(filename=os.path.join(dir, names[i]),
                                      location='npoi',
                                      ra=ra, dec=dec,
                                      format='ascii', exclude_t3amp=True))
    # data from VEGA
    dir = os.path.join(dir_data, 'vega')
    infile = os.path.join(dir, 'vega.ascii.lis')
    d = read_if(infile)
    fnames = d[:, 0]
    names = d[:, 1]
    for i in range(0, len(names)):
        print names[i]
        obs.append(pyshellspec.IFData(filename=os.path.join(dir, fnames[i]),
                                      location='chara',
                                      ra=ra, dec=dec,
                                      format='ascii'))
        obs[-1].set_filename(names[i])

    # construct data class
    data = pyshellspec.Data(obs)

    # construct the model
    cenobj = pyshellspec.CentralObject()
    compan = pyshellspec.Companion()
    disk = pyshellspec.Disk()
    orbit = pyshellspec.Orbit()
    spot = pyshellspec.Spot()
    objs = [cenobj, compan, disk, orbit, spot]
    model = pyshellspec.Model(objects=objs)

    # construct the Interface
    itf = pyshellspec.Interface(model=model, data=data, ncpu=4, image_size=180,
				if_phase_precision=3, lc_phase_precision=2, 
				if_ew_precision=7, lc_ew_precision=9,
				shellspec_template="template.in", use_offset=True)
    # set grid resolution
    itf.set_parameter('steps', value=1.0)
    itf.set_parameter('stepf', value=1.0)
    itf.set_parameter('stepsz', value=1.0)
    itf.set_parameter('stepfz', value=1.0)

    # set fitted parameter
    itf.set_parameter('idisc', value=2)  # slab
    itf.set_parameter('itdc', value=3)  # power-law
    itf.set_parameter('ispot', value=1)  # spot

    itf.set_parameter('ichemc', value=1)
    itf.set_parameter('ielnd', value=1)
    itf.set_parameter('ithom', value=0)
    itf.set_parameter('irayl', value=0)
    itf.set_parameter('imie', value=0)
    itf.set_parameter('imiepf', value=0)

    itf.set_parameter('asini', value=58.19, fitted=False, vmin=53., vmax=63.)

    itf.set_parameter('tempcp'  , value=13230.89688433398    , fitted=True, vmin=12000., vmax=14600.)
    itf.set_parameter('adisc'   , value=6.284220912553905    , fitted=True, vmin=5.5, vmax=12.)
    itf.set_parameter('routdc'  , value=31.04624426061674    , fitted=True, vmin=26., vmax=35.)
    itf.set_parameter('tempdc'  , value=33802.85992349101    , fitted=True, vmin=23000., vmax=34000.)
    itf.set_parameter('densdc'  , value=4.791360892674271e-06, fitted=True, vmin=5e-8, vmax=5e-6)
    itf.set_parameter('edendc'  , value=-0.8763764600823143  , fitted=True, vmin=-3.5, vmax=-0.8)
    itf.set_parameter('etmpdc'  , value=-1.038485121207513   , fitted=True, vmin=-1.2, vmax=-0.70)
    itf.set_parameter('dinc'    , value=93.54775419188991    , fitted=True, vmin=91., vmax=95.)
    itf.set_parameter('omega_an', value=253.4188207004175    , fitted=True, vmin=252., vmax=255.)
    itf.set_parameter('rsp'     , value=6.492617762518059    , fitted=True, vmin=4., vmax=2.0*6.36485904820971)
    itf.set_parameter('tempsp'  , value=9786.248399394492    , fitted=True, vmin=6000., vmax=25000.)
    itf.set_parameter('denssp'  , value=4.111211239016174e-08, fitted=True, vmin=1.e-11, vmax=1.e-7)
    itf.set_parameter('rpolsp'  , value=31.00445552977215    , fitted=True, vmin=20., vmax=31.09436071132663)
    itf.set_parameter('pangsp'  , value=0.6486123850475135   , fitted=True, vmin=-180., vmax=180.)
    itf.set_parameter('dd'      , value=313.9580689409981    , fitted=True, vmin=305., vmax=330.)

    itf.set_parameter('iline', value=1)

    # compute one fit
#    itf.compute_chi2(verbose=True)
#    itf.run_fit(fitter='sp_diff_evol', tol=1e-2, maxiter=10000)
    itf.run_fit(fitter='nlopt_nelder_mead', ftol=1e-6, maxiter=1000)

    itf.write_iterations()
    itf.set_model_to_shellspec()
    itf.write_template('final.in')
    itf.write_model()

    sys.exit(1) 
    files = [] 
    for o in obs:
        files.append(o.get_filename().split('/')[-1])
    for f in list(set(files)):
        itf.plot_comparison(filename=f)

if __name__ == '__main__':
    main()


